inline GEKMATRIX::GEKMATRIX(void)
{
	SetIdentity();
}

inline GEKMATRIX::GEKMATRIX(float *pfData)
{
	_11 = pfData[ 0];	_12 = pfData[ 1];	_13 = pfData[ 2];	_14 = pfData[ 3];
	_21 = pfData[ 4];	_22 = pfData[ 5];	_23 = pfData[ 6];	_24 = pfData[ 7];
	_31 = pfData[ 8];	_32 = pfData[ 9];	_33 = pfData[10];	_34 = pfData[11];
	_41 = pfData[12];	_42 = pfData[13];	_43 = pfData[14];	_44 = pfData[15];
}

inline GEKMATRIX::GEKMATRIX(float _11, float _12, float _13, float _14,
							float _21, float _22, float _23, float _24,
							float _31, float _32, float _33, float _34,
							float _41, float _42, float _43, float _44)
{
	this->_11 = _11;	this->_12 = _12;	this->_13 = _13;	this->_14 = _14;
	this->_21 = _21;	this->_22 = _22;	this->_23 = _23;	this->_24 = _24;
	this->_31 = _31;	this->_32 = _32;	this->_33 = _33;	this->_34 = _34;
	this->_41 = _41;	this->_42 = _42;	this->_43 = _43;	this->_44 = _44;
}

inline GEKMATRIX::GEKMATRIX(const GEKMATRIX &kMatrix)
{
	_11 = kMatrix._11;	_12 = kMatrix._12;	_13 = kMatrix._13;	_14 = kMatrix._14;
	_21 = kMatrix._21;	_22 = kMatrix._22;	_23 = kMatrix._23;	_24 = kMatrix._24;
	_31 = kMatrix._31;	_32 = kMatrix._32;	_33 = kMatrix._33;	_34 = kMatrix._34;
	_41 = kMatrix._41;	_42 = kMatrix._42;	_43 = kMatrix._43;	_44 = kMatrix._44;
}

inline GEKMATRIX::GEKMATRIX(const GEKVECTOR3 &kAxis, float fAngle)
{
	SetIdentity();
	SetRotation(kAxis, fAngle);
}

inline GEKMATRIX::GEKMATRIX(const GEKVECTOR3 &kEuler)
{
	SetIdentity();
	SetEuler(kEuler);
}

inline GEKMATRIX GEKMATRIX::operator = (const GEKMATRIX &kMatrix)
{
	_11 = kMatrix._11;	_12 = kMatrix._12;	_13 = kMatrix._13;	_14 = kMatrix._14;
	_21 = kMatrix._21;	_22 = kMatrix._22;	_23 = kMatrix._23;	_24 = kMatrix._24;
	_31 = kMatrix._31;	_32 = kMatrix._32;	_33 = kMatrix._33;	_34 = kMatrix._34;
	_41 = kMatrix._41;	_42 = kMatrix._42;	_43 = kMatrix._43;	_44 = kMatrix._44;
	return *this;
}

inline float GEKMATRIX::operator () (int iRow, int iCol) const
{
	return m[iRow][iCol];
}

inline float GEKMATRIX::operator () (int iRow, int iCol)
{
	return m[iRow][iCol];
}

inline GEKMATRIX::operator const float * (void) const
{
	return &_11;
}

inline GEKMATRIX::operator float * (void)
{
	return &_11;
}

inline void GEKMATRIX::SetIdentity(void)
{
	_11 = _22 = _33 = _44 = 1.0f;
	_12 = _13 = _14 = 0.0f;
	_21 = _23 = _24 = 0.0f;
	_31 = _32 = _34 = 0.0f;
	_41 = _42 = _43 = 0.0f;
}

inline void GEKMATRIX::SetScaling(const GEKVECTOR3 &kScale)
{
	_11 = kScale.x;
	_22 = kScale.y;
	_33 = kScale.z;
}

inline GEKVECTOR3 GEKMATRIX::GetScaling(void)
{
	return GEKVECTOR3(_11, _22, _33);
}

inline float GEKMATRIX::Determinant(void)
{
    return ((_11 * ((_22 * _33) - (_23 * _32))) -
            (_12 * ((_21 * _33) - (_23 * _31))) +
            (_13 * ((_21 * _32) - (_22 * _31))));
}