#include "..\GEKXMath.h"

void GEKFRUSTUM::Generate(const GEKMATRIX &kMatrix)
{
// Extract the near plane
	m_akFrustum[0].n.x = (kMatrix[ 3] + kMatrix[ 2]);
	m_akFrustum[0].n.y = (kMatrix[ 7] + kMatrix[ 6]);
	m_akFrustum[0].n.z = (kMatrix[11] + kMatrix[10]);
	m_akFrustum[0].d =   (kMatrix[15] + kMatrix[14]);
	
// Extract the far plane
	m_akFrustum[1].n.x = (kMatrix[ 3] - kMatrix[ 2]);
	m_akFrustum[1].n.y = (kMatrix[ 7] - kMatrix[ 6]);
	m_akFrustum[1].n.z = (kMatrix[11] - kMatrix[10]);
	m_akFrustum[1].d =   (kMatrix[15] - kMatrix[14]);

// Extract the left plane
	m_akFrustum[2].n.x = (kMatrix[ 3] + kMatrix[ 0]);
	m_akFrustum[2].n.y = (kMatrix[ 7] + kMatrix[ 4]);
	m_akFrustum[2].n.z = (kMatrix[11] + kMatrix[ 8]);
	m_akFrustum[2].d =   (kMatrix[15] + kMatrix[12]);

// Extract the right plane
	m_akFrustum[3].n.x = (kMatrix[ 3] - kMatrix[ 0]);
	m_akFrustum[3].n.y = (kMatrix[ 7] - kMatrix[ 4]);
	m_akFrustum[3].n.z = (kMatrix[11] - kMatrix[ 8]);
	m_akFrustum[3].d =   (kMatrix[15] - kMatrix[12]);

// Extract the bottom plane
	m_akFrustum[4].n.x = (kMatrix[ 3] + kMatrix[ 1]);
	m_akFrustum[4].n.y = (kMatrix[ 7] + kMatrix[ 5]);
	m_akFrustum[4].n.z = (kMatrix[11] + kMatrix[ 9]);
	m_akFrustum[4].d =   (kMatrix[15] + kMatrix[13]);

// Extract the top plane
	m_akFrustum[5].n.x = (kMatrix[ 3] - kMatrix[ 1]);
	m_akFrustum[5].n.y = (kMatrix[ 7] - kMatrix[ 5]);
	m_akFrustum[5].n.z = (kMatrix[11] - kMatrix[ 9]);
	m_akFrustum[5].d =   (kMatrix[15] - kMatrix[13]);
}

bool GEKFRUSTUM::Visible(const GEKVECTOR3 &kPoint) const
{
	for(int iIndex = 0; iIndex < 6; iIndex++)
	{
		if(m_akFrustum[iIndex].Distance(kPoint) <= 0.0f) return false;
	}

	return true;
}

bool GEKFRUSTUM::Visible(const GEKSPHERE &kSphere) const
{
	for(int iIndex = 0; iIndex < 6; iIndex++)
	{
		float fDistance = m_akFrustum[iIndex].Distance(kSphere.m_kOrigin);
		if(fDistance <= -kSphere.m_fRadius) return false;
	}

	return true;
}

bool GEKFRUSTUM::Visible(const GEKAABB &kAABB) const
{
	GEKVECTOR3 kPoint;
	for(int iIndex = 0; iIndex < 6; iIndex++)
	{
		kPoint.x = ((m_akFrustum[iIndex].n.x < 0.0f) ? kAABB.m_kMin.x : kAABB.m_kMax.x);
		kPoint.y = ((m_akFrustum[iIndex].n.y < 0.0f) ? kAABB.m_kMin.y : kAABB.m_kMax.y);
		kPoint.z = ((m_akFrustum[iIndex].n.z < 0.0f) ? kAABB.m_kMin.z : kAABB.m_kMax.z);
		if(m_akFrustum[iIndex].Distance(kPoint) <= 0.0f) return false;
	}

	return true;
}